/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file canny.cpp
/// @brief Example program for QVision library. Applies Canny operator over frames from input video.
///
/// This program will receive a path or url to a video file, and then will show the resulting video of aplying Canny operator
/// to it's frames.
/// To use the program, simply compile, and execute the binary:
///
///	./canny --URL=<path or url to video>
///
/// @author PARP Research Group. University of Murcia, Spain.
///

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <QDebug>

#include <qvcore/qvapplication.h>
#include <qvmplayercamera/qvmplayercamera.h>
#include <qvgui/qvisioninterface.h>
#include <qvipp/qvipp.h>

class CannyWorker: public QVWorker
	{
	public:
		CannyWorker(QString name): QVWorker(name)
			{
			addProperty<double>("Threshold high", inputFlag,	150,	"High threshold for Canny operator", 50, 1000);
			addProperty<double>("Threshold low", inputFlag,		50,	"Low threshold for Canny operator", 10, 500);
			addProperty< QVImage<uChar,1> >("Canny image", outputFlag);
			addProperty< QVImage<uChar,1> >("Input image", inputFlag|outputFlag);
			}

		void worker()
			{
			//////////////////
			// Read input parameters
			QVImage<uChar,1> image = getPropertyValue< QVImage<uChar,1> >("Input image");
			uInt cols = image.getCols(), rows = image.getRows();
			QVImage<sFloat>	imageFloat(cols, rows), dX(cols, rows), dY(cols, rows);
			QVImage<uChar>	canny(cols, rows), buffer;
		
			//////////////////
			// Convert image from uChar to sShort
			qvipp::Convert(image, imageFloat);
			timeFlag("Convert image from uChar to sShort");
		
			//////////////////
			// Obtain horizontal and vertical gradients from image
			qvipp::FilterSobelHorizMask(imageFloat,dY,3);
			qvipp::FilterSobelVertMask(imageFloat,dX,3);
			timeFlag("Obtain horizontal and vertical gradients from image");
		
			/////////////////
			// Apply Canny operator
			qvipp::CannyGetSize(canny, buffer);
			qvipp::Canny(dX, dY, canny, buffer, getPropertyValue<double>("Threshold low"), getPropertyValue<double>("Threshold high"));
			timeFlag("Apply Canny operator");
		
			/////////////////
			// Publish resulting images
			setPropertyValue< QVImage<uChar,1> >("Canny image",canny);
			timeFlag("Publish resulting images");
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		
		"Example program for QVision library. Shows an image fron a video file."

		);

	QVMPlayerCamera camera("Video");
	CannyWorker worker("Canny worker");
	QVisionInterface interface;

	camera.link(&worker,"Input image");
	interface.setVisible(&worker,"Input image");
	interface.setVisible(&worker,"Canny image");

	return app.exec();
	}
